'use strict';

const sass = require('node-sass');
const url = require('url');
const path = require('path');

module.exports = (options) => {
    const ext = options.ext !== '.sass' ? '.scss' : '.sass';
    const baseDir = options.baseDir || process.cwd();
    const inputDir = options.inputDir || null;
    const outputDir = options.outputDir || 'dist/css';
    const sourceMap = options.hasOwnProperty('sourceMap') ?
        (options.sourceMap !== false) : true;
    const includePaths = options.includePaths || [];
    const outputStyle = options.outputStyle || 'nested';
    const maxAge = options.maxAge || 0;
    const prefix = options.prefix || '';

    if (inputDir === null) {
        throw new Error('Cannot find inputDir');
    }

    function renderCss(req, res, next, reqPath) {
        let inputRealDir = path.resolve(baseDir, inputDir);
        let outputRealDir = path.resolve(baseDir, outputDir);
        let scssFile = path.join(inputRealDir, reqPath)
            .replace(/\.css$/, ext);
        let cssFile = path.join(outputRealDir, reqPath);
        let renderOptions = {
            file: scssFile,
            outFile: cssFile,
            includePaths:includePaths,
            outputStyle: outputStyle
        };

        if (sourceMap) {
            renderOptions.sourceMap = true;
            renderOptions.sourceMapContents = true;
        } else {
            renderOptions.omitSourceMapUrl = true;
        }

        sass.render(renderOptions, function (err, result) {
            if (err) {
                return next(err);
            }


            res.writeHead(200, {
                'Content-Type': 'text/css',
                'Cache-Control': 'max-age=' + maxAge
            });
            res.end(result.css);
        });
    }

    function renderSourceMap(req, res, next, reqPath) {
        let inputRealDir = path.resolve(baseDir, inputDir);
        let outputRealDir = path.resolve(baseDir, outputDir);
        let scssFile = path.join(inputRealDir, reqPath)
            .replace(/\.css\.map$/, ext);
        let cssFile = path.join(outputRealDir, reqPath)
            .replace(/\.css\.map$/, '.css');

        let renderOptions = {
            file: scssFile,
            outFile: cssFile,
            includePaths:includePaths,
            outputStyle: outputStyle,
            sourceMap: true,
            sourceMapContents: true
        };

        sass.render(renderOptions, function (err, result) {
            if (err) {
                return next(err);
            }


            res.writeHead(200, {
                'Cache-Control': 'max-age=' + maxAge
            });
            res.end(result.map);
        });
    }

    return (req, res, next) => {
        if (req.method !== 'GET' && req.method !== 'HEAD') {
            return next();
        }

        let reqPath = url.parse(req.url).pathname;

        if (prefix) {
            if (reqPath.indexOf(prefix === 0)) {
                reqPath = pauth.substring(prefix.length);
            } else {
                return next();
            }
        }

        if (/\.css$/.test(reqPath)) {
            return renderCss(req, res, next, reqPath);
        } else if (/\.css\.map$/.test(reqPath)) {
            return renderSourceMap(req, res, next, reqPath);
        } else {
            return next();
        }

    };
};
